-- landingquake, by Ashnal
-- Adds a sound and quake effect to hard landings
-- Inspired by Lat's Wipezones

if au_landingquake == nil then rawset (_G, "au_landingquake", true) else return end 

local FRACBITS = FRACBITS
local MFE_JUSTHITFLOOR = MFE_JUSTHITFLOOR
local sfx_doord2 = sfx_doord2
local sfx_s3k4c = sfx_s3k4c

local cv_landingquake_called = 0
local landingquake = CV_RegisterVar({
	name = "au_landingquake",
	defaultvalue = "Off",
	flags = CV_CALL,
	possiblevalue = CV_OnOff,
	func = function(var)
		cv_landingquake_called = $+1 -- We hackin this to not overwrite a local server/hostmod config that turns it off
		--print("landingquake called: "..cv_landingquake_called.." "..var.value)
	end
})

-- Set this to on for your server to default this on for joining players. They can however still turn it off using the above cvar at any time
local landingquakeserver = CV_RegisterVar({
	name = "au_landingquake_server",
	defaultvalue = "Off",
	flags = CV_NETVAR,
	possiblevalue = CV_OnOff,
})

-- This is at least how fast you must be falling, the kart's z momentum, to trigger the small bonk, in FU/T
local cv_small_threshold = CV_RegisterVar({
    name = "au_landingquake_small",
    defaultvalue = "24", -- Enough to trigger on spring penels when going full speed and medium-large ramps
    flags = CV_NETVAR,
    PossibleValue = {MIN = 0, MAX = 1000}
})

-- This is at least how fast you must be falling, the kart's z momentum, to trigger the louder thud and quake, in FU/T
local cv_large_threshold = CV_RegisterVar({
    name = "au_landingquake_large",
    defaultvalue = "48", -- Reserved for mostly large drops, like the end of lap drop on Grand Metropolis, or the huge stalling drop on Sand Valley
    flags = CV_NETVAR,
    PossibleValue = {MIN = 0, MAX = 1000}
})

addHook("ThinkFrame", function()
	if not consoleplayer then return end
	if cv_landingquake_called == 1 then -- should only be called once on load, it its been called again, that means the player has set it via config or manually wand we dont want to touch it
		COM_BufInsertText(consoleplayer, "au_landingquake "..landingquakeserver.value) -- or that we also touched it here
	end
end)

addHook("MobjThinker", function(mo)
    if landingquake.value and not mo.player.spectator and mo.previousmomz ~= mo.momz then
		if mo.eflags & MFE_JUSTHITFLOOR and not P_CheckDeathPitCollide(mo) then
			--print("floor hit momz: "..(abs(FixedMul(mo.previousmomz,mo.scale))>>FRACBITS))
			if abs(mo.previousmomz) > mo.scale*cv_large_threshold.value
				S_StartSound(mo, sfx_doord2)
				if not splitscreen and mo.player == displayplayers[0] then
					P_StartQuake(40<<FRACBITS, 6)
				end
			elseif abs(mo.previousmomz) > mo.scale*cv_small_threshold.value
				S_StartSound(mo, sfx_s3k4c)
			end
		end
		mo.previousmomz = mo.momz
	end
end, MT_PLAYER)